Package: GOstats
Title: Tools for manipulating GO and microarrays
Version: 2.77.0
Authors@R: c(
        person("Robert", "Gentleman", role="aut"),
        person("Seth", "Falcon", role="ctb"),
        person("Robert", "Castelo", role="ctb"),
        person("Sonali", "Kumari", role="ctb",
                comment="Converted vignettes from Sweave to R Markdown / HTML."),
        person("Dennis", "Ndubi", role="ctb",
                comment="Converted GOstatsHyperG vignette from Sweave to R Markdown / HTML."),
        person("Bioconductor Package Maintainer", role="cre",
                email="maintainer@bioconductor.org"))
Description: A set of tools for interacting with GO and microarray
        data. A variety of basic manipulation tools for graphs,
        hypothesis testing and other simple calculations.
Depends: R (>= 2.10), Biobase (>= 1.15.29), Category (>= 2.43.2), graph
Imports: methods, stats, stats4, AnnotationDbi (>= 0.0.89), GO.db (>=
        1.13.0), RBGL, annotate (>= 1.13.2), AnnotationForge, Rgraphviz
Suggests: hgu95av2.db (>= 1.13.0), ALL, multtest, genefilter,
        RColorBrewer, xtable, SparseM, GSEABase, geneplotter,
        org.Hs.eg.db, RUnit, BiocGenerics, BiocStyle, knitr
License: Artistic-2.0
Collate: zzz.R AllClasses.R AllGenerics.R hyperGTest-methods.R
        GOHyperGResult-accessors.R GOgraph.R GOhyptest.R hyperGtable.R
        shortestPath.R triad.R OBOHyperGResult-accessors.R
biocViews: Annotation, GO, MultipleComparison, GeneExpression,
        Microarray, Pathways, GeneSetEnrichment, GraphAndNetwork
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GOstats
git_branch: devel
git_last_commit: d09aa16
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 22:18:51 UTC; biocbuild
Author: Robert Gentleman [aut],
  Seth Falcon [ctb],
  Robert Castelo [ctb],
  Sonali Kumari [ctb] (Converted vignettes from Sweave to R Markdown /
    HTML.),
  Dennis Ndubi [ctb] (Converted GOstatsHyperG vignette from Sweave to R
    Markdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
